<?php
namespace Teknisa\CMP\Controllers;

use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Exception;
use Teknisa\CMP\Factories\Cmp99000GeracaoAfPrevisaoFactory;
use Teknisa\CMP\Services\Cmp99000GeracaoAfPrevisaoService;

class Cmp99000GeracaoAfPrevisaoController extends
    \Teknisa\GER\Controllers\TeknisaCrud
{
    protected $factory;
    protected $service;
    protected $crudService;
    protected $parameterBag;
    protected $environment;

    public function __construct(
        $crudService,
        $parameterBag,
        $environment,
        $factory,
        $service
    ) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->factory = $factory;
        $this->service = $service;
    }

    public function getGeracaoAfPrevisao(Row $request, Response $response)
    {
        try {
            $params = $request->getParameters();

            if ($params["NRAPURPREC"] == null) {
                $params["NRAPURPREC"] = $this->service->generateCode(
                    "APURPRECO" . $params["CDFILCENTCMP"],
                    $params["NRORG"]
                );

                $rowsAffect = $this->factory->gerarProcedure($params);
             
                if($rowsAffect['P_ROWSAFFECT'] == "N"){
                   $response->setError(new Error("Não há dados para Geração de AF.", 500));
                   
                   return;
                }
            }
            
            $data = $this->factory->getItmaPurPrecData($params);
            
            $response->addDataSet(new DataSet('data', $data));

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getSessionInfos(Row $request, Response $response)
    {
        $sessionInfos = [
            'CDFILIAL' => $this->environment->getCdFilial(),
            'CDOPERADOR' => $this->environment->getCdOperador()
        ];

        $response->addDataSet(new DataSet('data',  $sessionInfos));
    }
    
    public function updateProducts(Row $request, Response $response) {
        try {
            $rows = $request->getParameters();

            foreach ($rows as $row) {
                if(isset($row['QTSOLIPROD'])){
                    $row['originalRow']['QTSOLIPROD'] = $row['QTSOLIPROD'];
                }
                
                if(isset($row['VRCUSTO'])){
                    $row['originalRow']['VRCUSTO'] = $row['VRCUSTO'];
                }
    
                $this->factory->updateProducts($row['originalRow']);
            }
            
            $response->addDataSet(new DataSet('data', array( "success"  => true)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
     public function generateAfs(Row $request, Response $response) {
        try {
            $params = $request->getParameters();
          
            $rows = $params['rows'];
            
            $this->factory->beginTransaction();
 
            foreach ($rows as $key => $row) {
                if ($row['QTSOLIPROD'] <= 0 || $row['VRCUSTO'] <= 0) {
                    unset($rows[$key]);
    
                    continue;
                }
     
                $this->factory->updateProductStats($row);
            }
            
            if(count($rows) <= 0 ){
                $erroGeral = 'Z'; 
                $response->addDataSet(new DataSet('data', $erroGeral));
                     
                return;
            }
            
            $nrApurPrec = $rows[0]["NRAPURPREC"];
            $cdFillan = $rows[0]["CDFILLAN"];
            
            $efetivarAfParams = [
                "CDOPERADOR" => $params["CDOPERADOR"],
                "DTAUTOFORN" => date('d/m/y'),
                "CDFILLAN" => $cdFillan,
                "NRAPURPREC" => $nrApurPrec
            ];
         
            $procedure = $this->factory->efetivarAfProcedure($efetivarAfParams);
            $autFor = $this->factory->retornaAutFor($nrApurPrec, $cdFillan);
            
            $arrErro = array(); 
            $erroGeral = array('erroGeral' => 'N');
            
            foreach ($autFor as $row) {
                $erro = $this->factory->procedureFECHA_AUTOFORNE_MOBILE($row);
                $this->factory->editarStatusImportAutoForn($row);
                array_push($arrErro, $erro);
                if($erro['P_SEM_ERRO'] == 'N'){
                    $erroGeral = array('erroGeral' => 'S');
                }
            }
            array_push($arrErro, $erroGeral);
            
            $checkValida = $this->factory->getBlockedAF($autFor[0]);
            if($erroGeral['erroGeral'] == 'S'){
                if($checkValida['IDBLOQUEAAF'] == 'S'){
                    throw "inconsistente";
                }
            }

            $this->factory->editarStatusImportApurPreco($rows[0]);

            $response->addDataSet(new DataSet('data', $erroGeral));
            $this->factory->realizaCommit();
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->factory->realizaRollback();
        }
    }
}
